import os
import importlib
import binascii


def import_from_string(val):
    """
    Attempt to import a class from a string representation.
    """
    try:
        parts = val.split('.')
        module_path, class_name = '.'.join(parts[:-1]), parts[-1]
        module = importlib.import_module(module_path)
        return getattr(module, class_name)
    except (ImportError, AttributeError):
        raise ImportError("Could not import '%s'" % val)


def generate_key():
    return binascii.hexlify(os.urandom(20)).decode()



from apistar import Settings
from apistar.exceptions import ValidationError

from .compat import SqlalchemySession, DjangoSession
from .schemas import Login
from .settings import get_settings
from .utils import generate_key


def django_get_token(session: DjangoSession, settings: Settings,
                     data: Login):
    from django.contrib.auth import authenticate

    user_settings = get_settings(settings)
    user = authenticate(
        username=data['username'],
        password=data['password']
    )

    if not user:
        raise ValidationError(detail='Username or password invalid')
    
    TokenModel = getattr(session, user_settings['TOKEN_MODEL'])
    instance = TokenModel.objects.create(
        user=user, token=generate_key()
    )
    return {'token': instance.token}


def sqlalcamy_get_token(session: SqlalchemySession, settings: Settings,
                        data: Login):
    user_settings = get_settings(settings)
    username = data['username']
    password = user_settings['ENCRYPTION_FUNCTION'](data['password'])

    UserModel = user_settings['USER_MODEL']
    TokenModel = user_settings['TOKEN_MODEL']
    filters = {
        user_settings['USERNAME_FIELD']: username,
        user_settings['PASSWORD_FIELD']: password
    }
    user = session.query(UserModel).filter_by(**filters).first()

    if not user:
        raise ValidationError(detail='Username or password invalid')
    
    token = TokenModel(
        token=generate_key(), user_id=user.id
    )
    session.add(token)

    return {'token': token.token}



try:
    from apistar.backends.sqlalchemy_backend import Session as SqlalchemySession
except ImportError:
    SqlalchemySession = None

try:
    from apistar.backends.django_orm import Session as DjangoSession
except ImportError:
    DjangoSession = None



import datetime
import typing

from apistar import http, Settings
from apistar.authentication import Authenticated

from .compat import SqlalchemySession, DjangoSession
from .settings import get_settings


class BaseAuthentication():

    def get_credentials(self, authorization):
        if authorization is None:
            return None

        scheme, token = authorization.split()
        if scheme.title() != 'Token' or not token:
            return None

        return token


class SQLAlchemyTokenAuthentication(BaseAuthentication):

    def authenticate(self, authorization: http.Header,
                     session: SqlalchemySession,
                     settings: Settings) -> typing.Union[None, Authenticated]:
        user_settings = get_settings(settings)
        token = self.get_credentials(authorization)

        if not token:
            return

        TokenModel = user_settings['TOKEN_MODEL']
        UserModel = user_settings['USER_MODEL']
        instance = session.query(TokenModel).filter(TokenModel.token == token).first()
        if not instance:
            return
        
        now = datetime.datetime.now()
        difference = datetime.timedelta(days=user_settings['EXPIRY_TIME'])
        if user_settings['IS_EXPIRY_TOKEN'] and instance.created_at < (now - difference):
            return

        user = session.query(UserModel).filter(UserModel.id == instance.user_id).first()
        return Authenticated(
            username=getattr(user, user_settings['USERNAME_FIELD']),
            user=user
        )


class DjangoTokenAuthentication(BaseAuthentication):

    def authenticate(self, authorization: http.Header,
                     session: DjangoSession,
                     settings: Settings) -> typing.Union[None, Authenticated]:
        user_settings = get_settings(settings)
        token = self.get_credentials(authorization)

        if not token:
            return

        TokenModel = getattr(session, user_settings['TOKEN_MODEL'])
        UserModel = getattr(session, user_settings['USER_MODEL'])
        instance = TokenModel.objects.filter(token=token).first()
        if not instance:
            return

        now = datetime.datetime.now()
        difference = datetime.timedelta(days=user_settings['EXPIRY_TIME'])
        if user_settings['IS_EXPIRY_TOKEN'] and instance.created_at < (now - difference):
            return

        user = UserModel.objects.filter(id=instance.user_id).first()
        return Authenticated(
            username=getattr(user, user_settings['USERNAME_FIELD']),
            user=user
        )



from apistar import Settings
from apistar.exceptions import ConfigurationError

from .utils import import_from_string


SETTINGS_KEY = 'TOKEN_AUTHENTICATION'

DEFAULTS = {
    'IS_EXPIRY_TOKEN': False,
    'EXPIRY_TIME': 30,
    'USERNAME_FIELD': 'username',
    'PASSWORD_FIELD': 'password',
    'ORM': 'sqlalcamy',
    'ENCRYPTION_FUNCTION': lambda x: x
}

IMPORT_STRINGS = (
    'USER_MODEL',
    'TOKEN_MODEL',
    'ENCRYPTION_FUNCTION'
)


def get_settings(settings: Settings):
    user_settings = dict(DEFAULTS)
    user_settings.update(settings.get(SETTINGS_KEY, {}))

    for module in IMPORT_STRINGS:
        if module not in user_settings and module not in DEFAULTS:
            raise ConfigurationError(f'{module} settings is required.')

        if user_settings['ORM'] == 'sqlalcamy':
            user_settings[module] = import_from_string(user_settings[module])

    return user_settings



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup


setup(
    name='apistar_token_auth',
    version='1.0',
    packages=['apistar_token_auth'],
    url='https://github.com/bahattincinic/apistar_token_authentication',
    license='MIT',
    description='Token Based Authentication for API Star',
    author='Bahattin Cinic',
    author_email='bahattincinic@gmail.com',
    install_requires=['apistar'],
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Topic :: Internet :: WWW/HTTP',
    ]
)



from apistar import typesystem


class Password(typesystem.String):
    description = 'Password'


class Username(typesystem.String):
    description = 'Username'


class Login(typesystem.Object):
    properties = {
        'username': Username,
        'password': Password
    }
    required = ['username', 'password']
